#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <getopt.h>
#include <values.h>
#include <time.h>
#include <sys/time.h>
#include <cpgplot.h>
//#include <complex.h>
#include <fftw3.h>

#define MALLOC fftwf_malloc
//#define MALLOC malloc

#define MIN(x,y)   ((x) < (y) ? (x) : (y))

#define MAX(x,y)   ((x) > (y) ? (x) : (y))


double read_head(int file, struct tm *tm);

#define BLOCKSIZE 4*1024*1024

#define RATE 32
#define MAXSTR 256

typedef enum {VLBA, AT} encoding_type;

double tim(void) {
  struct timeval tv;
  double t;

  gettimeofday(&tv, NULL);
  t = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;

  return t;
}

#define MAXCHAN 8

int main (int argc, char * const argv[]) {
  int nfile, i, j, k, l, file, status, nread, opt, first, memsize, tmp;
  int naver, readshort, nfft, nx, ny, swapped, maxchan, nplot, bytesperint;
  register int samp;
  char msg[MAXSTR+1];
  unsigned char  *mem;
  unsigned short *mem16;
  off_t off, offset;
  float *in[MAXCHAN], lookup[4], *plotspec, *xvals, min, max, delta;
  fftwf_complex *out[MAXCHAN];
  double time, *spectrum[MAXCHAN], t0, t1, tt;
  long long lcount[4], tcount;
  struct tm tm;
  fftwf_plan plans[MAXCHAN]; /* Do up to MAXCHAN subbands at the same time */

  int verbose;       /* Prints whats happening */
  int bits;          /* Total number of bits/sample */
  int MHz64;         /* 64 MHz mode (4 consecutive samples/byte) */
  int nskip;         /* Number of bytes to skip at start of file */
  int nlags;         /* Number of lags to correlate */
  int chans[MAXCHAN];/* Channels tocorrelate */
  int bchan;         /* First spectral point to plot */
  int echan;         /* Last spectral point to plot */
  encoding_type encoding; /* VLBA or AT bit encoding (offset binary or sign/mag) */
  char pgdev[MAXSTR+1] = "/xs"; /* Pgplot device to use */

  int chan1, chan2;

  struct option options[] = {
    {"nlags", 1, 0, 'n'},
    {"skip", 1, 0, 's'},
    {"bits", 1, 0, 'b'},
    {"chan", 1, 0, 'c'},
    {"if1", 1, 0, '1'},
    {"if2", 1, 0, '2'},
    {"1", 1, 0, '1'},
    {"2", 1, 0, '2'},
    {"sp1", 1, 0, 'x'},
    {"sp2", 1, 0, 'y'},
    {"device", 1, 0, 'd'},
    {"pgdev", 1, 0, 'd'},
    {"vlba", 0, 0, 'v'},
    {"wide", 0, 0, 'w'},
    {"64", 0, 0, 'w'},
    {"verbose", 0, 0, 'V'},
    {"V", 0, 0, 'V'},
    {"help", 0, 0, 'h'},
    {0, 0, 0, 0}
  };

  verbose = 0;
  bits = 8;
  nskip = -1;
  nlags = 128;
  encoding = AT;
  MHz64 = 0;
  chan1 = -1;
  chan2 = -1;
  for (k=0;k<MAXCHAN;k++) chans[k]=0;
  bchan = -1;
  echan = -1;

  nfft = 0;
  /* Read command line options */
  while (1) {
    opt = getopt_long_only(argc, argv, "", options, NULL);
    if (opt==EOF) break;

    switch (opt) {
      
    case 'b':
      status = sscanf(optarg, "%d", &tmp);
      if (status!=1)
 	fprintf(stderr, "Bad -bits option %s\n", optarg);
      else 
	bits = tmp;
      break;

    case 'c':
      status = sscanf(optarg, "%d", &tmp);
      if (status!=1)
 	fprintf(stderr, "Bad -chan option %s\n", optarg);
      else {
	if (tmp>MAXCHAN) {
	  fprintf(stderr, "Chan %s too large\n", optarg);
	} else if (tmp<=0) {
	  fprintf(stderr, "Chan %s too small\n", optarg);
	} else {
	  chans[tmp-1] = 1;
	}
      }
      break;

    case '1':
      status = sscanf(optarg, "%d", &tmp);
      if (status!=1)
 	fprintf(stderr, "Bad -1chan option %s\n", optarg);
      else 
	chan1 = tmp-1;
      break;

    case '2':
      status = sscanf(optarg, "%d", &tmp);
      if (status!=1)
 	fprintf(stderr, "Bad -2chan option %s\n", optarg);
      else 
	chan2 = tmp;
      break;

    case 'x':
      status = sscanf(optarg, "%d", &tmp);
      if (status!=1)
 	fprintf(stderr, "Bad -x option %s\n", optarg);
      else 
	bchan = tmp;
      break;

    case 'y':
      status = sscanf(optarg, "%d", &tmp);
      if (status!=1)
 	fprintf(stderr, "Bad -z option %s\n", optarg);
      else 
	echan = tmp;
      break;

    case 's':
      fprintf(stderr, "Skip option not yet supported\n");
      status = sscanf(optarg, "%d", &tmp);
      if (status!=1)
 	fprintf(stderr, "Bad -skip option %s\n", optarg);
      else 
	nskip = tmp;
      break;

    case 'n':
      status = sscanf(optarg, "%d", &tmp);
      if (status!=1)
 	fprintf(stderr, "Bad -nlags option %s\n", optarg);
      else 
	nlags = tmp;
      break;

    case 'd':
      if (strlen(optarg)>MAXSTR) {
	fprintf(stderr, "PGDEV option %s too long\n", optarg);
	return 0;
      }
      strcpy(pgdev, optarg);
      break;

    case 'V':
      verbose = 1;
      break;

    case 'v':
      encoding = VLBA;
      break;

    case 'w':
      MHz64 = 1;
      break;

    case 'h':
      printf("Usage: vsib_checker [options]\n");
      printf("  -b/-bits        Total number bits/sample (4/8/16)\n");
      printf("  -s/-skip <n>    Skip this many bytes at start of file\n");
      printf("  -v/-vlba        Calculate statistics for VLBA data\n");
      printf("  -w/-wide/-64    Assume 64 MHz bandwidth data\n");
      printf("  -c/-chan <n>    Channel to correlate (can specify multipl times)\n");
      printf("  -if1/-1 <n>     1st channel to correlate in range\n");
      printf("  -if2/-2 <n>     Last channel to correlate in range\n");
      printf("  -sp1 <n>        First spectral point to plot\n");
      printf("  -sp2 <n>        Last spectral point to plot\n");
      printf("  -n/-nlags <n>   # lags to correlate\n");
      printf("  -device <pgdev> Pgplot device to plot to\n");
      printf("  -V/-verbose     Verbose\n");
      printf("  -h/-help        This list\n");
      return(1);
      break;

    case '?':
    default:
      break;
    }
  }

  /* Check too high channel hasn't been selected */
  maxchan = bits/2; 
  if (MHz64) maxchan /= 4;
  for (k=maxchan; k<MAXCHAN; k++) {
    if (chans[k]) {
      fprintf(stderr, "Channel %d is too large for this mode. Ignoring\n", k+1);
      chans[k] = 0;
    }
  }

  if (chan1>0 || chan2>0) {
    if (chan1<=0) 
      chan1=1;
    else if (chan1>maxchan) {
      fprintf(stderr, "Channel %d too large for current mode. Setting to %d\n",
	      chan1, maxchan);
      chan1=maxchan;
    }

    if (chan2<=0) 
      chan2=maxchan;
    else if (chan2>maxchan) {
      fprintf(stderr, "Channel %d too large for current mode. Setting to %d\n",
	      chan2, maxchan);
      chan2=maxchan;
    }
    if (chan2<chan1) {
      tmp = chan1;
      chan1 = chan2;
      chan2 = tmp;
    }
    for (k=chan1-1;k<chan2;k++) {
      chans[k] = 1;
    }
  }

  /* Have any channels been selected? */
  nplot = 0;
  for (k=0; k<maxchan; k++) {
    if (chans[k]) {
      nplot++;
    }
  }
  if (nplot==0) {
    fprintf(stderr, "Correlating channel 1\n");
    chans[0] = 1;
    nplot=1;
  }

  if (bchan<0) bchan = 0;
  if (echan<0) echan = nlags/2-1;
  if (bchan > nlags/2-1) bchan = nlags/2-1;
  if (echan > nlags/2-1) echan = nlags/2-1;
  if (bchan==echan) {
    fprintf(stderr, "Warning: Odd channel range. Resetting\n");
    bchan = 0;
    echan = nlags/2-1;
  }
  
  swapped = 0;
  if (bchan>echan) {
    swapped = bchan;
    bchan = echan;
    echan = swapped;
    swapped = 1;
    fprintf(stderr, "Swapping frequency axis\n");
  } 
  echan++;

  bytesperint = nlags*bits/8; /* Bytes per integration */
  if (MHz64) bytesperint /=4;  /* 4 samples per byte */
  printf("Bytes per integration = %d\n", bytesperint);

  memsize = floor(BLOCKSIZE/bytesperint)*bytesperint+0.1;

  mem = MALLOC(memsize);
  if (mem==NULL) {
    sprintf(msg, "Trying to allocate %d byes", memsize);
    perror(msg);
    return(1);
  }

  /* Initiate fftw */

  for (k=0; k<maxchan; k++) {
    if (!chans[k]) continue;
    in[k] = MALLOC(sizeof(float) * nlags);
    if (in[k]==NULL) {
      perror("Allocating memory");
      exit(1);
    }
    out[k] = MALLOC(sizeof(fftwf_complex) *nlags/2);
    if (out[k]==NULL) {
      perror("Allocating memory");
      exit(1);
    }
    spectrum[k] = MALLOC(sizeof(double)*nlags/2);
    if (spectrum[k]==NULL) {
      perror("Allocating memory");
      exit(1);
    }
    for (j=0; j<nlags/2; j++) {
      spectrum[k][j] = 0;
    }

    plans[k] = fftwf_plan_dft_r2c_1d(nlags, in[k], out[k], 0);
  }
  naver = 0;

  /* Setup the lookup table */
  if (encoding==VLBA) {
    lookup[0] = +3.0;
    lookup[1] = +1.0;
    lookup[2] = -1.0;
    lookup[3] = -3.0;
  } else { /* AT */
    lookup[0] = +1.0;
    lookup[1] = -1.0;
    lookup[2] = +3.0;
    lookup[3] = -3.0;
  }

  tt = 0; /* Total time */

  for (nfile=optind; nfile<argc; nfile++) {

    file = open(argv[nfile], O_RDONLY|O_LARGEFILE);
    if (file==-1) {
      sprintf(msg, "Failed to open input file (%s)", argv[nfile]);
      perror(msg);
      //fftwf_free(mem);
      return(1);
    }
    printf("%s\n", argv[nfile]);

    /* Read the header */
    time = read_head(file, &tm);

    for (i=0; i<4; i++) {
      lcount[i] = 0;
    }

    offset = 0;
    first = 1;
    readshort = 0;

    while (1) {
      nread = read(file, mem, memsize);

      if (nread==0) {
	if (offset==0) {
	  fprintf(stderr, "No data read from %s\n", argv[nfile]);
	  //fftwf_free(mem);
	  close(file);
	  return(1);
	}
	break;
      } else if (nread==-1) {
	perror("Error reading file");
	close(file);
	//fftwf_free(mem);
	return(1);
      } else if (nread%bytesperint != 0) { /* Need to read multiple of lags */
	/* Data may be pack with multiple samples/byte or mutiple 
	   bytes/sample */
	int shift;
	shift = nread % bytesperint;


	off = lseek(file, -shift, SEEK_CUR); 
	if (off==-1) {
	  perror(msg);
	  //fftwf_free(mem);
	  return(1);
	}
	nread -= shift;
      }
      offset += nread;
      t0 = tim();

      if (nread<bytesperint) { /* Not read enough */
	if (readshort) {
	  /* Probably at end of file */
	  if (offset < bytesperint) {
	    fprintf(stderr, "Warning: Short file\n");
	    exit(1);
	  } else {
	    break; 
	  }
	} else {
	  readshort = 1; /* Try one more time, just in case */
	  continue;
	}
      }

      mem16 = (unsigned short*)mem;

      /* The following loops decode the data into a set of float arrays */

      /* Copy data into "in" array, fft then accumulate */
      for (i=0; i<nread/bytesperint; i++) {
	if (MHz64) { /* 64 MHz needs to be handles seperatelt */
	  if (bits==8) {
	    fprintf(stderr, "64 MHz 8 bit not supported\n");
	    exit(1);
	  
	  } else if (bits==16) {
	    for (j=0; j<nlags/4; j++) {
	      for (k=0; k<maxchan; k++) {
		if (!chans[k]) continue;
		samp = mem[i*bytesperint+j*2+k];
		for (l=0; l<4; l++) {
		  in[k][j*4+l] = lookup[(samp>>(l*2))&0x3];
		  lcount[(samp>>(l*2))&0x3]++;
		}
	      }
	    }
	  }
	} else {
	  if (bits==8) {
	    for (j=0; j<nlags; j++) {
	      samp = mem[i*nlags+j];
	      for (k=0; k<maxchan; k++) {
		if (!chans[k]) continue;
		in[k][j] = lookup[(samp>>(k*2))&0x3];
		lcount[(samp>>(k*2))&0x3]++;
	      }
	    }

	  } else if (bits==16) {
	    for (j=0; j<nlags; j++) {
	      samp = mem16[i*nlags+j];
	      for (k=0; k<maxchan; k++) {
		if (!chans[k]) continue;
		in[k][j] = lookup[(samp>>(k*2))&0x3];
		lcount[(samp>>(k*2))&0x3]++;
	      }
	    }
	  } else if (bits==4) {
	    for (j=0; j<nlags/2; j++) {
	      samp = mem[i*bytesperint+j];
	      for (k=0; k<maxchan; k++) {
		if (!chans[k]) continue;
		in[k][j*2] = lookup[(samp>>(k*2))&0x3];
		in[k][j*2+1] = lookup[(samp>>(k*2+4))&0x3];
		lcount[(samp>>(k*2))&0x3]++;
		lcount[(samp>>(k*2+4))&0x3]++;
	      }
	    }
	  } else {
	    fprintf(stderr, "Unknown number of bits %d\n", bits);
	    exit(1);
	  }
	}

	for (k=0; k<maxchan; k++) {
	  if (!chans[k]) continue;

	  fftwf_execute(plans[k]);
	  nfft++;
	  for (j=0; j<nlags/2; j++) {
	    spectrum[k][j] += out[k][j][0]*out[k][j][0] 
	      + out[k][j][1]*out[k][j][1];
	  }
	}
	naver++;
	}
      t1 = tim();
      tt += t1-t0;
    }
  }

  fftwf_free(mem);
  for (k=0; k<maxchan; k++) {
    if (!chans[k]) continue;
    //fftwf_destroy_plan(plans[k]);
    //fftwf_free(in[k]); 
    //fftwf_free(out[k]);
  }

  printf("Total computational time= %0.1f seconds for %d ffts\n", 
	 tt, nfft);

  xvals = MALLOC(sizeof(float)*nlags/2);
  for (j=0; j<nlags/2; j++) {
    xvals[j] = j+0.5;
  }

  /* How many plots to do */;
  ny = ceil(sqrt(nplot));
  nx = (nplot+ny-1)/ny;

  if (cpgbeg(0,pgdev,nx,ny)!=1) {
    fprintf(stderr, "Error calling PGBEGIN");
    return(1);
  }

  plotspec = MALLOC(sizeof(float)*nlags/2);

  for (k=0; k<maxchan; k++) {
    if (!chans[k]) continue;

    max = 0;
    min = 0;

    for (j=bchan; j<echan; j++) {
      plotspec[j] = spectrum[k][j] / naver;
      if (plotspec[j]>max) 
	max = plotspec[j];
      else if (plotspec[j]<min) 
	min = plotspec[j];
    }

    delta = (max-min)*0.05;
    if (min<0) min -= delta;
    max += delta;

    cpgsci(7);
    if (swapped) 
      cpgenv(echan,bchan,min,max,0,0);
    else 
      cpgenv(bchan,echan,min,max,0,0);
    sprintf(msg, "Channel %d", k+1);
    cpglab("Channel", "Correlation", msg);
    cpgsci(2);
    cpgline(echan-bchan, xvals, &plotspec[bchan]);

    //fftwf_free(spectrum[k]);
    //fftwf_free(plotspec[k]);

  }
  fftwf_free(xvals);

  tcount = 0;
  for (i=0; i<4; i++) {
    tcount += lcount[i];
  }
  printf("Total counts %.1fe6\n", tcount/1e6);
  for (i=0; i<4; i++) {
    printf(" %5.1f", lcount[i]/(double)tcount*100);
  }
  printf("\n");

  cpgend();

  return(1);
}
