// This program receives a file through UDT
// Usage: recvfile server_ip server_port filename filesize

#include <iostream>
#include <stdlib.h>
#include <udt.h>

int main(int argc, char* argv[])
{
  if (sizeof(long long)!=8) {
    cout << "Error: sizeof(long long) must equal 8" << endl;
  }

  if ((argc != 4) || (0 == atoi(argv[2])))
   {
      cout << "usage: hclient server_ip server_port filename" << endl;
      return 0;
   }

   CUDT* sender = new CUDT;

   try
   {
      sender->open();
      sender->connect(argv[1], atoi(argv[2]));
   }
   catch(CUDTException e)
   {
      cout << "error message: " << e.getErrorMessage();
      return 0;
   }
   cerr << "Connected" << endl;


   try
   {
     int intval;
     bool boolval;
     int vallen;

     boolval = true;
     sender->setOpt(UDT_SNDSYN, &boolval, sizeof(bool));


      //can be called anywhere after step 1.
      // read UDT configurations
      sender->getOpt(UDT_PORT, &intval, vallen);
      cout << "current port number: " << intval << endl;
      sender->getOpt(UDT_PCH, &boolval, vallen);
      cout << "current port number changable?: " << boolval << endl;

      sender->getOpt(UDT_SNDSYN, &boolval, vallen);
      cout << "Sending is blocking?: " << boolval << endl;
      sender->getOpt(UDT_RCVSYN, &boolval, vallen);
      cout << "Receiving is blocking?: " << boolval << endl;
      sender->getOpt(UDT_MFLAG, &intval, vallen);
      cout << "Sent buffer is auto-released?: " << intval << endl;

      sender->getOpt(UDT_FC, &intval, vallen);
      cout << "Maximum flow control window size: " << intval << endl;

      sender->getOpt(UDT_BUF, &intval, vallen);
      cout << "UDT buffer size: " << intval << endl;

      sender->getOpt(UDT_USB, &intval, vallen);
      cout << "UDP sending buffer size: " << intval << endl;

      sender->getOpt(UDT_URB, &intval, vallen);
      cout << "UDP receiving buffer size: " << intval << endl;

      sender->getOpt(UDT_IPV, &intval, vallen);
      cout << "Using IP version: " << "IPv" << intval << endl;
   }
   catch (CUDTException e)
   {
      cout << "error msg: " << e.getErrorMessage();
      return 0;
   }

   ifstream ifs(argv[3]);

   ifs.seekg(0, ios::end);
   long long size = ifs.tellg();
   ifs.seekg(0, ios::beg);

   cout << "Sending file sized " << size << endl;

   //try {
   //  long long size2 = size;
   //  sender->send((char *)&size2, 8);
   //}
   //catch (CUDTException e) {
   //  cout << "Error message: " << e.getErrorMessage();
   // }

   timeval t1, t2;

   gettimeofday(&t1, 0);

   try
   {
      sender->sendfile(ifs, 0, size);
   }
   catch (CUDTException e)
   {
      cout << "error message: " << e.getErrorMessage();
      return 0;
   }

   sender->close();
   delete sender;

   gettimeofday(&t2, 0);

   cout << "speed = " << (double(size) * 8. / 1000000.) / (t2.tv_sec - t1.tv_sec + (t2.tv_usec - t1.tv_usec) / 1000000.) << "Mbits/sec" << endl;


   return 1;
}
